@@ -1,3 +1,4 @@
+import argparse
 import sys
 import os
 from distutils.dir_util import copy_tree
@@ -5,74 +6,44 @@ import pathlib
 import shutil
 import platform
 import subprocess
-import time
-import threading
-import urllib.request
-import json
+from lsbatch.constants import *
+from lsbatch.requests_handler import RequestHandler
+from lsbatch.batch_job_operations import BatchJobOperations
+from lsbatch.helper import Helper, ProgressBar
+from lsbatch.log import Log
 
-current_version = '1.0.0'
-virtual_env_name = 'batch-virtualenv'
+current_version = CURRENT_VERSION
+virtual_env_name = VIRTUAL_ENV_NAME
 command_run_python = ''
 command_activate_virual_env = ''
 run_in_shell = False
 file_path = pathlib.Path(__file__).resolve()
 
 
-class ProgressBar(threading.Thread):
-    def __init__(self, message, stop=False, kill=False):
-        super().__init__()
-        self.stop = stop
-        self.kill = kill
-        self.message = message
-
-    def run(self):
-        print(self.message)
-        sys.stdout.flush()
-        i = 0
-        while not (self.stop or self.kill):
-            if (i % 4) == 0:
-                sys.stdout.write('\b/')
-            elif (i % 4) == 1:
-                sys.stdout.write('\b-')
-            elif (i % 4) == 2:
-                sys.stdout.write('\b\\')
-            elif (i % 4) == 3:
-                sys.stdout.write('\b|')
-
-            sys.stdout.flush()
-            time.sleep(0.2)
-            i += 1
-        if self.kill:
-            print('\b\b\b\b ABORT!')
-        else:
-            print('\b\b\b done!')
-
-
 def main():
     try:
         setup_commands()
         args = sys.argv[1:]
         script_path = os.getcwd()
         if len(args) == 0:
-            print('Which task? Values: (init/pack/run/install/uninstall/help):', end=' ')
+            Log.question(Messages.SelectTask)
             sys.argv.append(input())
             main()
         elif args[0] == 'init':
             initiate_batch(script_path)
         elif args[0] == 'pack':
-            if validate_lsbatch_pack(script_path):
-                shutil.make_archive(os.path.join(script_path, os.path.basename(script_path)), 'zip',
-                                    script_path + '/src')
+            if Helper.validate_lsbatch_pack(script_path):
+                Helper.zip_source_code(script_path)
         elif args[0] == 'help':
             print_help_section()
         elif args[0] == 'run':
             if os.path.exists(os.path.join(script_path, 'startup.py')):
                 run_batch_job(script_path)
             else:
-                print('startup.py file is not present in ' + script_path)
+                Log.error(Messages.StartUpFileNotPresent.format(script_path))
         elif args[0] == 'install':
             if len(args) > 2:
-                print('lsbatch install can take 1 argument, python package name')
+                Log.error(ValidationMessages.LSBatchInstallArguments)
                 return
             if len(args) == 1:
                 lsbatch_install(script_path)
@@ -80,36 +51,85 @@ def main():
             lsbatch_install_package(script_path, args[1])
         elif args[0] == 'uninstall':
             if len(args) != 2:
-                print('lsbatch uninstall takes 1 argument, python package name')
+                Log.error(ValidationMessages.LSBatchUninstallArguments)
                 return
             uninstall_package(script_path, args[1])
+        elif args[0] == 'save':
+            credentials = get_developer_api_credentials_from_arg(args[1:], 'save', ['settings'])
+            if credentials is None:
+                return
+            BatchJobOperations.lsbatch_save(script_path, credentials['settings'])
+        elif args[0] == 'publish':
+            credentials = get_developer_api_credentials_from_arg(args[1:], 'publish')
+            if credentials is None:
+                return
+            BatchJobOperations.lsbatch_publish(script_path)
+        elif args[0] == 'configure':
+            BatchJobOperations.lsbatch_configure()
         else:
-            print('lsbatch takes 1 argument, (init/pack/run/install/uninstall/help)')
+            Log.error(ValidationMessages.LSBatchArguments)
     finally:
         try:
             latest_version = get_latest_version()
             if not (current_version == latest_version):
-                print(
-                    'WARNING! new version of lsbatch is available, please upgrade using "pip install --upgrade lsbatch"')
+                Log.info(Messages.LatestVersionWarning)
         except Exception as ex:
             pass
 
 
+def get_developer_api_credentials_from_arg(args, sub_command, additional_bool_arguments=None):
+    parser = argparse.ArgumentParser(description=f'lsbatch {sub_command}', prog=f'lsbatch {sub_command}')
+    parser.add_argument('-o', '--orgcode', dest=Constants.OrgCode, type=str, required=False)
+    parser.add_argument('-t', '--token', dest=Constants.Token, type=str, required=False)
+    parser.add_argument('-r', '--region', dest=Constants.Region, type=str, help=Messages.RegionArgumentHelp, required=False)
+    parser.add_argument('-p', '--profile', dest=Constants.Profile, type=str, required=False)
+
+    if additional_bool_arguments is not None:
+        for arg in additional_bool_arguments:
+            parser.add_argument(f'-{arg[0]}', f'--{arg}', action='store_true')
+
+    credentials = parser.parse_args(args)
+    credentials_dict = credentials.__dict__
+    error_message = None
+    if credentials_dict.get(Constants.Profile) is not None:
+        if credentials_dict[Constants.Profile] == '':
+            error_message = ValidationMessages.ProfileNameEmpty
+        else:
+            profile_credentials = BatchJobOperations.retrieve_configuration(credentials_dict[Constants.Profile])
+            if profile_credentials is None:
+                error_message = ValidationMessages.ProfileNameNotFound
+            else:
+                credentials_dict.update(profile_credentials)
+    elif credentials_dict.get(Constants.OrgCode) is None or credentials_dict.get(
+            Constants.Token) is None or credentials_dict.get(Constants.Region) is None:
+        error_message = ValidationMessages.ArgumentsNotPassedCorrectly
+
+    if error_message is None:
+        error_message = Helper.validate_credentials(credentials_dict, True)
+        if error_message is None:
+            RequestHandler.set_up_credentials(credentials_dict)
+            return credentials_dict
+    Log.error(error_message)
+
+
 def initiate_batch(script_path):
     try:
         import virtualenv
     except ImportError:
-        print('Can not find virtualenv, please install virtualenv, run "pip install virtualenv"')
+        Log.error(ValidationMessages.VirtualEnvironmentNotFound)
         return
-    print('Your Batch Job name:', end=' ')
+    Log.question(Messages.YourBatchName)
     batch_job_name = input()
-    progress_bar = ProgressBar(message='Installing dependencies')
+    progress_bar = ProgressBar(message=Messages.InstallingDependencies)
     progress_bar.start()
     try:
-        copy_tree(str(file_path.parent / 'lsq_batch_template'),
-                  script_path + '/' + batch_job_name)
-        add_gitignore(os.path.join(script_path, batch_job_name))
-        returncode = create_virtual_environment(os.path.join(script_path, batch_job_name))
+        batch_job_directory = os.path.join(script_path, batch_job_name)
+        copy_tree(os.path.join(file_path.parent, 'lsq_batch_template'), batch_job_directory)
+        add_gitignore(batch_job_directory)
+        batch_details = Helper.retrieve_batch_details(batch_job_directory)
+        batch_details["Name"] = batch_job_name
+        Helper.update_batch_details(batch_job_directory, batch_details)
+        returncode = create_virtual_environment(batch_job_directory)
         if not (returncode == 0):
             progress_bar.kill = True
             return
@@ -123,12 +143,12 @@ def initiate_batch(script_path):
 
 def lsbatch_install(script_path):
     if os.path.exists(os.path.join(script_path, 'src')):
-        progress_bar = ProgressBar(message='Installing dependencies....')
+        progress_bar = ProgressBar(message=Messages.InstallingDependencies)
         progress_bar.start()
         try:
             batch_template_path = os.path.join(file_path.parent, 'lsq_batch_template')
             for filename in os.listdir(batch_template_path):
-                files_not_to_overrride = ['event.json', 'query.json', 'settings.json', 'sample_query_result.csv']
+                files_not_to_overrride = ['event.json', 'query.json', 'settings.json', 'sample_query_result.csv', 'batch_details.json']
                 if not (filename == 'src' or filename == '__pycache__' or (
                         filename in files_not_to_overrride and os.path.exists(os.path.join(script_path, filename)))):
                     if os.path.isdir(os.path.join(batch_template_path, filename)):
@@ -144,7 +164,7 @@ def lsbatch_install(script_path):
             requirements_file_path = os.path.join(script_path, 'src', 'requirements.txt')
             if os.path.exists(requirements_file_path):
                 process = subprocess.Popen(
-                    '{} && pip install -r {}'.format(command_activate_virual_env, requirements_file_path),
+                    '{} && pip install -r "{}"'.format(command_activate_virual_env, requirements_file_path),
                     shell=run_in_shell, stdout=subprocess.PIPE)
                 process.communicate()
                 if not (process.returncode == 0):
@@ -157,7 +177,7 @@ def lsbatch_install(script_path):
             progress_bar.kill = True
             raise ex
     else:
-        print('To install batch job dependencies current directory should have src folder which resides user code')
+        Log.error(ValidationMessages.NotFoundSourceFolder)
 
 
 def remove_package_requirements_txt(path, package_name):
@@ -175,16 +195,16 @@ def lsbatch_install_package(script_path, package):
         package_and_version = package.split("==")
         reqirements_path = os.path.join(script_path, 'src', 'requirements.txt')
         if package_in_default_packages(package_and_version[0]):
-            print('Requirement already satisfied')
+            Log.info(Messages.RequirementSatisfied)
             return
         if os.path.exists(reqirements_path):
             with open(reqirements_path) as f:
                 data = f.readlines()
                 for line in data:
                     if package + '\n' == line.lower() or package == line.lower():
-                        print('Requirement already satisfied')
+                        Log.info(Messages.RequirementSatisfied)
                         return
-        process = subprocess.Popen('{} && pip install {}'.format(command_activate_virual_env, package),
+        process = subprocess.Popen('{} && pip install "{}"'.format(command_activate_virual_env, package),
                                    shell=run_in_shell)
         process.wait()
         if process.returncode == 0:
@@ -193,31 +213,17 @@ def lsbatch_install_package(script_path, package):
             with open(reqirements_path, mode='a') as requirement_file:
                 requirement_file.write(package + '\n')
     else:
-        print(virtual_env_name + ' not present in directory ' + script_path)
-
-
-def validate_lsbatch_pack(script_path):
-    if not os.path.exists(os.path.join(script_path, 'src')):
-        print(os.path.join(script_path, 'src') + ' is not a directory')
-        return False
-    if not os.path.exists(os.path.join(script_path, 'src', 'main.py')):
-        print('main.py file is not present in ' + os.path.join(script_path, 'src'))
-        return False
-    with open(os.path.join(script_path, 'src', 'main.py')) as py_file:
-        if not ('def main(' in py_file.read()):
-            print('main function is not present in ' + os.path.join(script_path, 'src', 'main.py'))
-            return False
-    return True
+        Log.error(virtual_env_name + ' not present in directory ' + script_path)
 
 
 def print_help_section():
-    print("To init a batch job: lsbatch init\n To pack a batch job: lsbatch pack\n To run a batch job: lsbatch run")
+    Log.info(Messages.LSBatchHelpSection)
 
 
 def create_virtual_environment(path):
     os.chdir(path)
-    command_install_packages = 'pip install -r {}'.format(
-        str(file_path.parent / 'virtual-env-requirements.txt'))
+    command_install_packages = 'pip install -r "{}"'.format(
+        os.path.join(file_path.parent, 'virtual-env-requirements.txt'))
     command_create_virtul_env = '{} -m venv {}'.format(command_run_python, virtual_env_name)
     process2 = subprocess.Popen(command_create_virtul_env, stdout=subprocess.PIPE, shell=run_in_shell)
     process2.communicate()
@@ -232,7 +238,7 @@ def create_virtual_environment(path):
 
 
 def package_in_default_packages(package_name):
-    default_requirement_file = file_path.parent / 'virtual-env-requirements.txt'
+    default_requirement_file = os.path.join(file_path.parent, 'virtual-env-requirements.txt')
     with open(default_requirement_file, 'r') as f:
         data = f.readlines()
     for line in data:
@@ -245,9 +251,9 @@ def uninstall_package(script_path, package):
     package_and_version = package.split("==")
     if os.path.exists(os.path.join(script_path, virtual_env_name)):
         if package_in_default_packages(package_and_version[0]):
-            print('Can not uninstall this package, this package is a part of default packages')
+            Log.error(ValidationMessages.CanNotUninstallDefaultPackage)
             return
-        process = subprocess.Popen('{} && pip uninstall {}'.format(command_activate_virual_env, package),
+        process = subprocess.Popen('{} && pip uninstall "{}"'.format(command_activate_virual_env, package),
                                    shell=run_in_shell)
         process.wait()
         if process.returncode == 0:
@@ -255,7 +261,7 @@ def uninstall_package(script_path, package):
             if os.path.exists(requirement_path):
                 remove_package_requirements_txt(requirement_path, package_and_version[0])
     else:
-        print(virtual_env_name + ' not present in directory ' + script_path)
+        Log.error(virtual_env_name + ' not present in directory ' + script_path)
 
 
 def run_batch_job(activation_path):
@@ -266,8 +272,9 @@ def run_batch_job(activation_path):
 
 
 def add_gitignore(path):
-    if os.path.exists(file_path.parent / 'gitignore.txt'):
-        with open(file_path.parent / 'gitignore.txt', 'r') as gitignore_file:
+    gitignore_file_path = os.path.join(file_path.parent, 'gitignore.txt')
+    if os.path.exists(gitignore_file_path):
+        with open(gitignore_file_path, 'r') as gitignore_file:
             gitignore_file_content = gitignore_file.read()
 
         with open(os.path.join(path, '.gitignore'), 'w') as gitignore_file:
@@ -275,8 +282,7 @@ def add_gitignore(path):
 
 
 def get_latest_version():
-    contents = urllib.request.urlopen('https://pypi.org/pypi/lsbatch/json').read()
-    data = json.loads(contents)
+    data = RequestHandler.get_package_info()
     latest_version = data['info']['version']
     return latest_version
 
