@@ -24,7 +24,7 @@ def get_event():
     event['source'] = event_json['source'] if 'source' in event_json else {}
     event['job_name'] = os.path.basename(pathlib.Path(__file__).resolve().parent)
     event['job_instance_id'] = uuid.uuid4().__str__()
-    event['retry_attempt'] = 1
+    event['retry_attempt'] = event_json['retry_attempt'] if 'retry_attempt' in event_json else 0
     return event
 
 
